"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const websocket_1 = require("websocket");
class CfNotificationsService {
    constructor(config) {
        this.dontRetry = false;
        this.config = config;
        console.log('starting CfNotificationsService');
        this.setDefaultConfig();
    }
    /**
     * open connection to the WebSocket end point
     */
    watch() {
        return new rxjs_1.Observable(obser => {
            console.log(`watch for data from  ${this.config.endPoint}/notifications`);
            this.connect(obser);
        });
    }
    close() {
        this.dontRetry = true;
        this.ws.close();
    }
    send(data) {
        this.ws.send(data);
    }
    connect(obser) {
        this.log('connecting to WebSocket');
        this.ws = new websocket_1.w3cwebsocket(`${this.config.endPoint}`);
        // will run when the user will unsubscribe
        obser.add(() => {
            this.close();
        });
        this.ws.onopen = () => {
            obser.next({
                type: 'start'
            });
        };
        this.ws.onerror = (event) => {
            obser.next({
                type: 'error',
                event
            });
            this.retry(obser);
        };
        this.ws.onclose = (event) => {
            obser.next({
                type: 'close',
                event
            });
            this.retry(obser);
        };
        this.ws.onmessage = (event) => {
            obser.next({
                type: 'message',
                event
            });
        };
    }
    retry(obser) {
        if (this.dontRetry) {
            this.dontRetry = false;
            return;
        }
        clearTimeout(this.delayRetry);
        this.delayRetry = setTimeout(() => {
            this.log('try to reconnect to WebSocket');
            this.connect(obser);
        }, this.config.reconnectDelay);
    }
    setDefaultConfig() {
        if (!this.config.debug)
            this.config.debug = true;
        if (!this.config.endPoint)
            throw 'Error -> Missing endPoint value';
        if (!this.config.reconnectDelay)
            this.config.reconnectDelay = 5000;
    }
    log(message) {
        if (!this.config.debug)
            return;
        console.log(message);
    }
}
exports.CfNotificationsService = CfNotificationsService;
//# sourceMappingURL=cfNotifications.service.js.map